CLEAR
CLEARALL
@ 4,0
TEXT
               Metaphone and Soundex from Steve Skiles, 72310,433

         From time to time there are enquiries made on the forum about the
      Soundex algorithm and using it in PAL. Dan Paolini posted an imple-
      mentation of that algorithm some time ago. Enclosed is a somewhat
      different implementation. Also enclosed is an implementation of the
      Metaphone algorithm and a small test program to allow the comparison
      of the two. Metaphone seems to have two advantages over Soundex:

          1. It produces code strings which are somewhat readable, and
          2. It is more discriminating since it takes each letter's phone-
             tic context into account.

               Press [Esc] to Quit, or any other key to continue.
ENDTEXT
n=GETCHAR()
IF n = 27 THEN
  QUIT
ENDIF
CLEAR
@ 2,0
TEXT
      METAPHONE is an algorithm which is alledged to be better than Soundex
   for detecting homonyms.  The PAL procedure presented here is adapted from
   Computer Language Magazine 7.12 Dec 90 pp. 39-43, "Hanging on the Meta-
   phone" by Lawrence Phillips. Phillips wrote in PICK Basic, a dialect which
   (to me at least) is not easily readable. In addition, there is a major
   typo and ommission of text in the coding for the treatment of "G". I have
   tried to translate the procedure into PAL, taking advantage of the PAL
   vocabulary to make things more efficient and less complex.

      Phillips uses single characters to represent 16 consonant sounds:

                          B F H J K L M N P R S T W Y
                             X for "sh" sounds and
                             0 for "th" sounds.

      Phillips' procedure was one giant routine which made the algorithm a
   bit more clear. I have broken some parts of it out into separate routines
   in order to take advantage of the PAL commands, EXECUTE and EXECPROC.

               Press [Esc] to Quit, or any other key to continue.
ENDTEXT
n=GETCHAR()
IF n = 27 THEN
  QUIT
ENDIF
CLEAR
@ 4,0
TEXT
      This procedure seems useful to me beyond being an exercise in learning
   PAL. I would appreciate any suggestions for improvement in efficiency,
   accuracy or comprehensiveness.

      There remain, of course, anomalies in the codings produced (e.g. some
   hard CH's produce X instead of K), and I would much appreciate reports of
   any howlers.

      I release my work to the public domain (you'll have to ask CLM about
   theirs), so use and enjoy.

         Steve Skiles, 72310,433

               Press [Esc] to Quit, or any other key to continue.
ENDTEXT
n=GETCHAR()
IF n = 27 THEN
  QUIT
ENDIF


;============================================================================
;
;        TITLE: Metaphone!C.u()
;
;  DESCRIPTION: Process the letter C for Metaphone.a
;       AUTHOR: Stephen H. Skiles, based on a PICK Basic procedure by
;               Lawrence Phillips.
;      CREATED: 3 May 92
; LAST REVISED:
;
;         TYPE: Utility subroutine
;   PARAMETERS: None
;      RETURNS: Nothing
;    CALLED BY: Metaphone.a via EXECPROC
;        CALLS: Nothing
;
;        NOTES:
;
;============================================================================
;
PROC Metaphone!C.u()
  PRIVATE a

  a = ThisChar.a
  SWITCH
    CASE LastChar.a = "S" AND                    ; Skip C in SCI, SCE or SCY
         NextChar.a <> "" AND
         SEARCH(NextChar.a, Soften.a) > 0 :
      a = ""                        
    CASE NextChar.a = "I" AND                    ; CI = SH = X in CIA
         ThrdChar.a = "A" :
      a = "X"                       
    CASE NextChar.a <> "" AND                    ; C = S in CE, CI or CY
         SEARCH(NextChar.a, Soften.a) > 0 :
      a = "S"                         
    CASE NextChar.a = "H" :                      ; CH..
      SWITCH
        CASE LastChar.a = "S" :                  ;   C = K in ..SCH..
          a = "K"
        CASE LastChar.a = "" :                   ;   Initial CH
          IF ThrdChar.a <> "" AND
             SEARCH(ThrdChar.a, Vowels.a) > 0 THEN
            a = "X"                              ;     + vowel = SH = X
          ELSE
            a = "K"                              ;     + consonant = K
          ENDIF
        OTHERWISE : a = "X"                      ;   Else CH = SH = X
      ENDSWITCH
    OTHERWISE : a = "K"                          ; Otherwise C = K
  ENDSWITCH
  Ret.a = Ret.a + a
ENDPROC
; WRITELIB LibName.a Metaphone!C.u
; RELEASE  PROCS     Metaphone!C.u
; ?? ""


;============================================================================
;
;        TITLE: Metaphone!D.u()
;
;  DESCRIPTION: Process the letter D for Metaphone.a
;       AUTHOR: Stephen H. Skiles, based on a PICK Basic procedure by
;               Lawrence Phillips.
;      CREATED: 3 May 92
; LAST REVISED:
;
;         TYPE: Utility subroutine
;   PARAMETERS: None
;      RETURNS: Nothing
;    CALLED BY: Metaphone.a via EXECPROC
;        CALLS: Nothing
;
;        NOTES:
;
;============================================================================
;
PROC Metaphone!D.u()
  IF NextChar.a = "G" AND
     ThrdChar.a <> "" AND
     SEARCH(ThrdChar.a, Soften.a) > 0 THEN
    Ret.a = Ret.a + "J"                          ; DG + vowel = J
  ELSE
    Ret.a = Ret.a + "T"                          ; otherwise D = T
  ENDIF
ENDPROC
; WRITELIB LibName.a Metaphone!D.u
; RELEASE  PROCS     Metaphone!D.u
; ?? ""


;============================================================================
;
;        TITLE: Metaphone!G.u()
;
;  DESCRIPTION: Process the letter G for Metaphone.a
;       AUTHOR: Stephen H. Skiles, based on a PICK Basic procedure by
;               Lawrence Phillips.
;      CREATED: 3 May 92
; LAST REVISED:
;
;         TYPE: Utility subroutine
;   PARAMETERS: None
;      RETURNS: Nothing
;    CALLED BY: Metaphone.a via EXECPROC
;        CALLS: Nothing
;
;        NOTES:
;
;============================================================================
;
PROC Metaphone!G.u()
  PRIVATE a

  a = ThisChar.a
  Hard.l = False
  SWITCH                                         ; Skip silent G
    CASE NextChar.a = "H" :
      IF ThrdChar.a <> "" AND
         SEARCH(ThrdChar.a, Vowels.a) = 0 THEN                 ; Skip GH@
        a = ""
      ENDIF
    CASE NextChar.a = "N" :
      SWITCH
        CASE ThrdChar.a = ""                  : a = ""         ; Skip final GN
        CASE SEARCH(ThrdChar.a, Soften.a) > 0 : a = ""         ; Skip GN{E,I,Y}
        CASE SEARCH(ThrdChar.a, Vowels.a) > 0 : Hard.l = True  ; Save GN{A,O,U}
        OTHERWISE                             : a = ""         ; Skip other GN
      ENDSWITCH
    CASE NextChar.a <> "" :                                    ; Skip DG{E,I,Y}
      IF LastChar.a = "D" AND
         SEARCH(NextChar.a, Soften.a) > 0 THEN
        a = ""
      ENDIF
  ENDSWITCH
  IF LastChar.a = "G" THEN                       ; Second G in GG is hard
    Hard.l = True
  ENDIF
  IF NextChar.a <> "" AND                        ; Soft G = J
     SEARCH(NextChar.a, Soften.a) > 0 AND
     NOT Hard.l THEN
    a = "J"
  ELSE                                           ; Hard G = K
    a = "K"
  ENDIF
  Ret.a = Ret.a + a
ENDPROC
; WRITELIB LibName.a Metaphone!G.u
; RELEASE  PROCS     Metaphone!G.u
; ?? ""


;============================================================================
;
;        TITLE: Metaphone!H.u()
;
;  DESCRIPTION: Process the letter H for Metaphone.a
;       AUTHOR: Stephen H. Skiles, based on a PICK Basic procedure by
;               Lawrence Phillips.
;      CREATED: 3 May 92
; LAST REVISED:
;
;         TYPE: Utility subroutine
;   PARAMETERS: None
;      RETURNS: Nothing
;    CALLED BY: Metaphone.a via EXECPROC
;        CALLS: Nothing
;
;        NOTES:
;
;============================================================================
;
PROC Metaphone!H.u()
  SWITCH
    CASE NextChar.a = ""                   : ; Skip final H
    CASE LastChar.a <> "" AND
         SEARCH(LastChar.a, Spirant.a) > 0 : ; Skip spirant H
    OTHERWISE                              : Ret.a = Ret.a + ThisChar.a
  ENDSWITCH
ENDPROC
; WRITELIB LibName.a Metaphone!H.u
; RELEASE  PROCS     Metaphone!H.u
; ?? ""


;============================================================================
;
;        TITLE: Metaphone!S.u()
;
;  DESCRIPTION: Process the letter S for Metaphone.a
;       AUTHOR: Stephen H. Skiles, based on a PICK Basic procedure by
;               Lawrence Phillips.
;      CREATED: 3 May 92
; LAST REVISED:
;
;         TYPE: Utility subroutine
;   PARAMETERS: None
;      RETURNS: Nothing
;    CALLED BY: Metaphone.a via EXECPROC
;        CALLS: Nothing
;
;        NOTES:
;
;============================================================================
;
PROC Metaphone!S.u()
  SWITCH
    CASE LastChar.a <> "" AND
         NextChar.a = "I" AND
         (ThrdChar.a = "O" OR ThrdChar.a = "A") :
      Ret.a = Ret.a + "X"                        ; S = SH = X in @SIO or @SIA
    CASE NextChar.a = "H" :
      Ret.a = Ret.a + "X"                        ; S = SH = X in SH
    OTHERWISE :
      Ret.a = Ret.a + ThisChar.a
  ENDSWITCH
ENDPROC
; WRITELIB LibName.a Metaphone!S.u
; RELEASE  PROCS     Metaphone!S.u
; ?? ""


;============================================================================
;
;        TITLE: Metaphone!T.u()
;
;  DESCRIPTION: Process the letter T for Metaphone.a
;       AUTHOR: Stephen H. Skiles, based on a PICK Basic procedure by
;               Lawrence Phillips.
;      CREATED: 3 May 92
; LAST REVISED:
;
;         TYPE: Utility subroutine
;   PARAMETERS: None
;      RETURNS: Nothing
;    CALLED BY: Metaphone.a via EXECPROC
;        CALLS: Nothing
;
;        NOTES:
;
;============================================================================
;
PROC Metaphone!T.u()
  SWITCH
    CASE LastChar.a <> "" AND
         NextChar.a = "I" AND
         (ThrdChar.a = "O" OR ThrdChar.a = "A") :
      Ret.a = Ret.a + "X"                        ; T = SH = X in @TIO or @TIA
    CASE LastChar.a <> "T" AND
         NextChar.a = "H" :
      Ret.a = Ret.a + "0"                        ; T = 0 in TH except after T
    CASE NextChar.a = "C" AND
         ThrdChar.a = "H" :
      RETURN                                     ; Silent T in TCH
    OTHERWISE : Ret.a = Ret.a + ThisChar.a
  ENDSWITCH
ENDPROC
; WRITELIB LibName.a Metaphone!T.u
; RELEASE  PROCS     Metaphone!T.u
; ?? ""


;============================================================================
;
;        TITLE: Metaphone.a(Source.a, MaxLen.n, Pad.l)
;
;  DESCRIPTION: Return a phonetic code string for Source.a. Codes produced by
;               different strings sound alike if they are equal.
;       AUTHOR: Stephen H. Skiles, based on a PICK Basic procedure by
;               Lawrence Phillips.
;      CREATED: 5 Dec 90
; LAST REVISED: 3 May 92
;
;         TYPE: Utility
;   PARAMETERS: Source.a  the string to be encoded
;               MaxLen.n  the maximum length of the returned string, OR
;                         0 to encode all of Source.a
;               Pad.l     TRUE   pad the returned string with "*" to MaxLen.n
;                         FALSE  do not pad the returned string
;      RETURNS: A string consisting of the phonetic encoding of Source.a
;               If MaxLen.n > 0, the returned string will be of length,
;               MaxLen.n or MaxLen.n + 1 (if the last letter processed
;               produces a digraph code).
;    CALLED BY:
;        CALLS:
;
;        NOTES: In the comments, "c" stands for any consonant, "v" stands for
;               any vowel, "@" stands for any one letter, and "x" stands for
;               any zero or more letters. Upper-case letters are litteral,
;               upper-case letters.
;
;============================================================================
;
PROC Metaphone.a(Source.a, MaxLen.n, Pad.l)
  PRIVATE UtilProc.a,                       ; Procedure name for debugging
          Vowels.a, Soften.a, Spirant.a,    ; Constant strings
          Letter.r,                         ; Dispatch table
          LastChar.a, ThisChar.a,           ; Characters considered
          NextChar.a, ThrdChar.a,
          Hard.l,                           ; Logical flag
          Ret.a,                            ; Return value
          RetLen.n, SourceLen.n,            ; Control variables
          a, n, l                           ; Temporary variables

  UtilProc.a  = "Metaphone.a"                    ; INITIALIZE CONSTANTS
  Vowels.a    = "AEIOU"                          ; Normal vowels
  Soften.a    = "EIY"                            ; Softens preceding consonant
  Spirant.a   = "CSPTG"                          ; Changed by following H
  ARRAY Letter.r[26]                             ; Process dispatch table A-Z
    Letter.r[1]  = "Ret.a=Ret.a+\"A\""
    Letter.r[2]  = "IF NOT(LastChar.a=\"M\" AND NextChar.a=\"\") "+
                   "THEN Ret.a=Ret.a+\"B\" ENDIF"
    Letter.r[3]  = "C"
    Letter.r[4]  = "D"
    Letter.r[5]  = "Ret.a=Ret.a+\"E\""
    Letter.r[6]  = "Ret.a=Ret.a+\"F\""
    Letter.r[7]  = "G"
    Letter.r[8]  = "H"
    Letter.r[9]  = "Ret.a=Ret.a+\"I\""
    Letter.r[10] = "Ret.a=Ret.a+\"J\""
    Letter.r[11] = "IF LastChar.a<>\"C\" "+
                   "THEN Ret.a=Ret.a+\"K\" ENDIF"
    Letter.r[12] = "Ret.a=Ret.a+\"L\""
    Letter.r[13] = "Ret.a=Ret.a+\"M\""
    Letter.r[14] = "Ret.a=Ret.a+\"N\""
    Letter.r[15] = "Ret.a=Ret.a+\"O\""
    Letter.r[16] = "IF NextChar.a=\"H\" "+
                   "THEN Ret.a=Ret.a+\"F\" "+
                   "ELSE Ret.a=Ret.a+\"P\" ENDIF"
    Letter.r[17] = "Ret.a=Ret.a+\"K\""
    Letter.r[18] = "Ret.a=Ret.a+\"R\""
    Letter.r[19] = "S"
    Letter.r[20] = "T"
    Letter.r[21] = "Ret.a=Ret.a+\"U\""
    Letter.r[22] = "Ret.a=Ret.a+\"F\""
    Letter.r[23] = "IF NOT(NextChar.a=\"\" OR SEARCH(NextChar.a,Vowels.a)=0) "+
                   "THEN Ret.a=Ret.a+\"W\" ENDIF"
    Letter.r[24] = "Ret.a=Ret.a+\"KS\""
    Letter.r[25] = "IF NOT(NextChar.a=\"\" OR SEARCH(NextChar.a,Vowels.a)=0) "+
                   "THEN Ret.a=Ret.a+\"Y\" ENDIF"
    Letter.r[26] = "Ret.a=Ret.a+\"S\""

                                                 ; NORMALIZE Source.a
  Source.a = UPPER(Source.a)                     ; Use only upper case
                                                 ; Adjust initial 2 chars
  l = MATCH(Source.a, "@@..", ThisChar.a, NextChar.a, Source.a)
  IF l THEN
    a = ThisChar.a + NextChar.a                       ; Assume no change
    SWITCH
      CASE ThisChar.a = "X"     : a = "S" + NextChar.a     ; Initial X = S
      CASE NextChar.a = "N"     :                          ; Silent P,K or G
        IF SEARCH(ThisChar.a, "PKG") > 0 THEN              ;   before N
          a = NextChar.a
        ENDIF
      CASE a = "AE" OR a = "WR" : a = NextChar.a           ; AE = E, WR = R
      CASE a = "WH"             : a = ThisChar.a           ; WH = W
    ENDSWITCH
    Source.a = a + Source.a                           ; Rebuild Source.a
  ENDIF

  SourceLen.n = LEN(Source.a)                    ; INITIALIZE FOR LOOP
  IF MaxLen.n < 1 THEN
    MaxLen.n = SourceLen.n
    Pad.l  = FALSE                               ; Can't pad if MaxLen.n = 0
  ENDIF
  ThisChar.a = ""
  NextChar.a = SUBSTR(Source.a, 1, 1)
  ThrdChar.a = SUBSTR(Source.a, 2, 1)
  Ret.a = ""
  RetLen.n = 0

  FOR n FROM 1 TO SourceLen.n                    ; MAIN PROCESSING LOOP
    IF RetLen.n >= MaxLen.n                      ; Quit if code is long enough
      THEN QUITLOOP
    ENDIF
    LastChar.a = ThisChar.a
    ThisChar.a = NextChar.a                      ; Move along in source string
    NextChar.a = ThrdChar.a
    ThrdChar.a = SUBSTR(Source.a, n + 2, 1)
    SWITCH
      CASE ThisChar.a < "A" OR ThisChar.a > "Z" :     ; Skip non-alpha chars
        ThisChar.a = LastChar.a
        LOOP
      CASE n > 1 :
        IF ((ThisChar.a <> "C") AND                   ; Skip duplicate chars
            (LastChar.a = ThisChar.a)) OR             ;   except CC
           SEARCH(ThisChar.a, Vowels.a) > 0 THEN      ; Skip non-initial vowels
          LOOP 
        ENDIF
    ENDSWITCH
    a = Letter.r[ASC(ThisChar.a) - 64]           ; Get letter's command string
    IF SEARCH("=", a) > 0 THEN
      EXECUTE a                                  ; Execute assignments
    ELSE
      EXECPROC "Metaphone!" + a + ".u"           ; Call procedures
    ENDIF
    RetLen.n = LEN(Ret.a)                        ; May be 0,1 or 2 chars longer
  ENDFOR                                         ; END MAIN PROCESSING LOOP

  IF RetLen.n >= MaxLen.n OR NOT Pad.l THEN      ; RETURN THE CODE
    RETURN Ret.a                                 ;   Plain
  ELSE
    RETURN Ret.a + FILL("*", MaxLen.n - RetLen.n);   Padded
  ENDIF
ENDPROC
; WRITELIB LibName.a Metaphone.a
; RELEASE  PROCS     Metaphone.a
; ?? ""


;============================================================================
;
;        TITLE: Soundex.a(Source.a, MaxLen.n, Pad.l)
;
;  DESCRIPTION: Compute the Soundex equivalent of a string.
;       AUTHOR: Stephen H. Skiles, based on a public-domain Clarion procedure
;               by Paul Giroux (72007,3677).
;      CREATED: Sat  05-02-1992  at 21:54:28
; LAST REVISED:
;
;         TYPE: Utility
;   PARAMETERS: Source.a  the string to be processed.
;               MaxLen.n  the desired length of the return string (the longer
;                         you want, the longer it takes). If Length.n is zero,
;                         all of Source.a will be processed.
;               Pad.l     TRUE   to pad return string to MaxLen.n with "*"
;                         FALSE  to omit padding
;      RETURNS: A string of length, MaxLen.n, consisting of the first letter
;               of Source.a and a series of numeral equivalents of the other
;               alphabetic characters in the string, e.g.:
;
;               Source.a MaxLen.n Pad.l Returns|Source.a MaxLen.n Pad.l Returns
;                                              |
;               ""          0     N/A   ""     |"AEIOU"     0     N/A   ""
;               ""          3     TRUE  "***"  |"AEIOU"     3     TRUE  "A**"
;               ""          3     FALSE ""     |"AEIOU"     3     FALSE "A"
;               "123#%"     0     N/A   ""     |"TWERP"     0     N/A   "T61"
;               "123#%"     3     TRUE  "***"  |"TWERP"     5     TRUE  "T61**"
;               "123#%"     3     FALSE ""     |"TWERP"     5     FALSE "T61"
;                                              |
;    CALLED BY:
;        CALLS:
;
;        NOTES: Due to reduction by the algorithm, the length of the returned
;               string may be less than that requested, but it will never be
;               longer. If Pad.l is TRUE, the length of the returned string
;               will be as requested.
;
;               Soundex is almost useless if you cannot be sure of the first
;               character. A returned string of only Soundex digits is too
;               imprecise a predictor of homonymity (i.e. "jazzy" would be
;               declared to sound like "wacky").
;
;============================================================================
;
PROC Soundex.a(Source.a, MaxLen.n, Pad.l)
  PRIVATE UtilProc.a,        ; Proc name for debugging
          Convert.a,         ; Constant conversion string
          Ret.a,             ; Return string
          RetLen.n,          ; Length of return string
          ThisChar.a,        ; Current character of interest
          LastChar.a,        ; Character before ThisChar.a
          l, n, a            ; Temporary variables

  UtilProc.a = "Soundex.a"                       ; INITIALIZE
  Convert.a = "01230120022455012623010202"       ; Soundex equivalent string
            ;  ABCDEFGHIJKLMNOPQRSTUVWXYZ
  Source.a = UPPER(Source.a)                     ; Upper case only
  IF MaxLen.n < 1 THEN
    MaxLen.n = LEN(Source.a)
    Pad.l = FALSE                                ; Can't pad if MaxLen.n = 0
  ENDIF
  Ret.a   = ""
  RetLen.n = 0
  LastChar.a = ""
  l = TRUE
  WHILE l                                        ; LOOP THROUGH Source.a
    l = MATCH(Source.a, "@..", ThisChar.a, Source.a);Get next char
    IF ((NOT l) OR (ThisChar.a = LastChar.a)) THEN  ;Skip double letters
      LOOP                                          ;or Source.a = ""
    ENDIF
    n  = ASC(ThisChar.a) - 64                    ;   ASCII value = substr index
    IF ((n < 1) OR (n > 26)) THEN                ;   Skip non-alphabetic chars
       LOOP        
    ENDIF
    a = SUBSTR(Convert.a, n, 1)                  ;   Get conversion
    SWITCH
      CASE RetLen.n = 0   : Ret.a = ThisChar.a   ;   1st char always used
      CASE a = "0"        : LOOP                 ;   Insignificant char
      OTHERWISE           : Ret.a = Ret.a + a    ;   Use soundex digit
    ENDSWITCH
    RetLen.n = RetLen.n + 1                      ;   Increment Ret.a length
    IF RetLen.n = MaxLen.n THEN                  ;   Long enough?
      QUITLOOP
    ENDIF
    LastChar.a = ThisChar.a                      ;   Reset preceding char
  ENDWHILE
  IF RetLen.n = MaxLen.n OR NOT Pad.l THEN       ; RETURN
    RETURN Ret.a                                      ; Plain return string
  ELSE
    RETURN Ret.a + FILL("*", MaxLen.n - RetLen.n)     ; Padded return string
  ENDIF
ENDPROC
; WRITELIB LibName.a Soundex.a
; RELEASE  PROCS     Soundex.a
; ?? ""


; Test.u      Test procedure for the Soundex.a and Metaphone.a procedures
;
PROC Test.u()
  PRIVATE CurrentProc.a,     ; Name of proc for debugging
          RetLen.s,          ; Procedure call parameter
          Pad.l,             ; Procedure call parameter
          Source1.a,         ; Procedure call parameter
          Source2.a,         ; Procedure call parameter
          Soundex1.a,        ; Soundex returned string
          Soundex2.a,        ; Soundex returned string
          Metaphone1.a,      ; Metaphone returned string
          Metaphone2.a,      ; Metaphone returned string
          n                  ; Temporary variable (used with GETCHAR())

  CurrentProc.a = "Test.u"
  CLEAR                                          ; Set display screen
  @ 2,0
TEXT
   ͻ
                     COMPARISON  OF  HOMONYM  ALGORITHMS                  
   ͹
     PARAMETERS   Return Length:         (0 to 25)           Press [Esc]  
                   Asterisk Pad:         (True or False)         To Quit  
   ͹
     CODES      First Source String          Second Source String       
       
        Source                                                          
       
       Soundex                                                          
     Metaphone                                                          
       
                      Soundex Says:                                       
                    Metaphone Says:                                       
   Ķ
                      Press [Esc] to change parameters.                   
   ͼ
ENDTEXT
  WHILE TRUE                                     ; Loop 'til cancelled
    PAINTCANVAS FILL " "  5,34, 6,39                       ; Clear param area
    @ 5,34 ACCEPT "S" MIN 0 MAX 25 REQUIRED TO RetLen.s    ; Get MaxLen.n
    IF NOT RETVAL THEN
      QUITLOOP
    ENDIF
    @ 6,34 ACCEPT "A5" PICTURE "{TRUE,FALSE}" REQUIRED TO Pad.l ; Get Pad.l
    IF NOT RETVAL THEN
      QUITLOOP
    ENDIF
    IF Pad.l = "TRUE" THEN                                 ; String --> Logic
      Pad.l = TRUE
    ELSE
      Pad.l = FALSE
    ENDIF
    WHILE TRUE                                        ; Loop 'til Esc
      PAINTCANVAS FILL " " 10,18,10,42                  ; Clear params
      PAINTCANVAS FILL " " 10,48,10,72
      PAINTCANVAS FILL " " 12,18,13,42                  ; Clear results
      PAINTCANVAS FILL " " 12,48,13,72
      PAINTCANVAS FILL " " 15,37,16,52
      @ 10,18 ACCEPT "A25" TO Source1.a                 ; Get source strings
      IF NOT RETVAL THEN
        QUITLOOP
      ENDIF
      @ 10,48 ACCEPT "A25" TO Source2.a
      IF NOT RETVAL THEN
        QUITLOOP
      ENDIF
      Soundex1.a = Soundex.a(Source1.a,RetLen.s,Pad.l)  ; Get returned strings
      Soundex2.a = Soundex.a(Source2.a,RetLen.s,Pad.l)
      Metaphone1.a = Metaphone.a(Source1.a,RetLen.s,Pad.l)
      Metaphone2.a = Metaphone.a(Source2.a,RetLen.s,Pad.l)
      @ 12,18                                           ; Display returns
      IF Soundex1.a = "" THEN
        ?? " Blank String "
      ELSE
        ?? Soundex1.a
      ENDIF
      @ 12,48
      IF Soundex2.a = "" THEN
        ?? " Blank String "
      ELSE
        ?? Soundex2.a
      ENDIF
      @ 13,18
      IF Metaphone1.a = "" THEN
        ?? " Blank String "
      ELSE
        ?? Metaphone1.a
      ENDIF
      @ 13,48
      IF Metaphone2.a = "" THEN
        ?? " Blank String "
      ELSE
        ?? Metaphone2.a
      ENDIF
      @ 15,37
      IF Soundex1.a = Soundex2.a THEN
        ?? "Sound ALIKE"
      ELSE
        ?? "Sound DIFFERENT"
      ENDIF
      @ 16,37
      IF Metaphone1.a = Metaphone2.a THEN
        ?? "Sound ALIKE"
      ELSE
        ?? "Sound DIFFERENT"
      ENDIF
      n = GETCHAR()
      IF n = 27 THEN
        QUITLOOP
      ENDIF
    ENDWHILE                                          ; Esc to parameters
  ENDWHILE                                       ; Esc to script
ENDPROC


Test.u()                                         ; Test the procedures
RELEASE PROCS ALL
RELEASE VARS  ALL


